{
  Voice Modem Libriary v1.0

  This file contain definition of the simple log file.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Log;

interface

uses
  Windows, SysUtils, Classes;

type

{ TModemLogFile }

  {
    Simple log file. Open exists log (text) file or creates new file, move to
    the file end and write log title. Then ready to add log lines.

    Procedures:

    Add       - Add line to log file.

    Properties:

    CreateNew - Create new log file, erase old file.
    FileName  - Log file name. 
    Title     - Log title.
  }

  TModemLogFile = class(TComponent)
  private
    FFileName: string;
    FTitle: string;
    FLogFile: TFileStream;
    FCreateNew: Boolean;
    procedure SetFileName(Value: string);
  public
    destructor Destroy; override;
    procedure Add(const S: string);
  published
    property CretateNew: Boolean read FCreateNew write FCreateNew default False;
    property FileName: string read FFileName write SetFileName;
    property Title: string read FTitle write FTitle;
  end;

implementation

{ TModemLogFile }

destructor TModemLogFile.Destroy;
begin
  FLogFile.Free;
  inherited Destroy;
end;

procedure TModemLogFile.SetFileName(Value: string);
begin
  if FFileName <> Value then
  begin
    FFileName := Value;
    { Close log file }
    FLogFile.Free;
    FLogFile := nil;
  end;
end;

procedure TModemLogFile.Add(const S: string);
begin
  try
    { Open log file }
    if FLogFile = nil then
    begin
      { Open log file }
      if not FCreateNew then
      begin
        { Check for new file }
        if not FileExists(FFileName) then
        begin
          FLogFile := TFileStream.Create(FFileName, fmCreate);
          FLogFile.Free;
        end;
        { Open log file, allow other users read log }
        FLogFile := TFileStream.Create(FFileName, fmOpenWrite or fmShareDenyWrite);
        { Move to end of file end write empty line }
        if FLogFile.Size > 0 then
        begin
          FLogFile.Position := FLogFile.Size;
          FLogFile.Write(#13#10, 2);
        end;
      end
      else
      begin
        { Create new file }
        FLogFile := TFileStream.Create(FFileName, fmCreate);
        FLogFile.Free;
        { Open log file, allow other users read log }
        FLogFile := TFileStream.Create(FFileName, fmOpenWrite or fmShareDenyWrite);
      end;
      { Write Title }
      if Length(FTitle) > 0 then
      begin
        FLogFile.Write(Pointer(FTitle)^, Length(FTitle));
        FLogFile.Write(#13#10, 2);
        FLogFile.Write(#13#10, 2);
      end;
    end;
    { Write line }
    FLogFile.Write(Pointer(S)^, Length(S));
    FLogFile.Write(#13#10, 2);
  except
    { Ignore all exceptions }
  end;
end;

end.

