{
  Voice Modem Libriary v1.0

  This file contains the ZyXEL 2864 specific hardware stuff.

  Based on a original source code by Martin Seine <martin@erde.gun.de>,
  that used the 2864 compatible mode of the Elite.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_ZyXEL2864;

interface

uses
  Windows, SysUtils, Classes, VML_Modem, VML_Consts, VML_IS101;

const

{ ZyXEL 2864 available compression modes }

  COMPRESSION_ZYXEL2864_ADPCM_2    = 2;
  COMPRESSION_ZYXEL2864_ADPCM_3    = 3;
  COMPRESSION_ZYXEL2864_ADPCM_30   = 30;
  COMPRESSION_ZYXEL2864_ADPCM_4    = 4;
  COMPRESSION_ZYXEL2864_DVIADPCM_4 = 40;
  COMPRESSION_ZYXEL2864_ALAW_PCM   = 80;
  COMPRESSION_ZYXEL2864_MULAW_PCM  = 81;

type

{ TZyXEL2864 }

  TZyXEL2864 = class(TIS101)
  private
    FROMRelease: Integer;
  public
    function AnswerPhone: Integer; override;
    class function GetCompression(const Name: string): Integer; override;
    class function GetCompressionName(Compression: Integer): string; override;
    class function GetCompressionsList(Proc: TGetStrProc): Integer; override;
    class function GetDevice(const Name: string): Integer; override;
    class function GetDeviceName(Device: Integer): string; override;
    class function GetDevicesList(Proc: TGetStrProc): Integer; override;
    class function GetPlayDevicesList(Proc: TGetStrProc): Integer; override;
    class function GetRecordDevicesList(Proc: TGetStrProc): Integer; override;
    function Init: Integer; override;
    function InitVoice: Integer; override;
    class function ModemName: string; override;
    class function ModemRMDName: string; override;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer; override;
    function SetDevice(Device: Integer): Integer; override;
    property ROMRelease: Integer read FROMRelease;
  end;

implementation

uses
  VML_Idents;

{$I *.INC}

{ Some internal constants }

const
  CompressionMap: array[0..6] of TIdentMapEntry = (
    (Value: COMPRESSION_ZYXEL2864_ADPCM_2; Name: SADPCM2),
    (Value: COMPRESSION_ZYXEL2864_ADPCM_3; Name: SADPCM3),
    (Value: COMPRESSION_ZYXEL2864_ADPCM_30; Name: SADPCM30),
    (Value: COMPRESSION_ZYXEL2864_ADPCM_4; Name: SADPCM4),
    (Value: COMPRESSION_ZYXEL2864_DVIADPCM_4; Name: SDVIADPCM4),
    (Value: COMPRESSION_ZYXEL2864_ALAW_PCM; Name: SALAWPCM),
    (Value: COMPRESSION_ZYXEL2864_MULAW_PCM; Name: SMULAWPCM));

  DeviceMap: array[0..3] of TIdentMapEntry = (
    (Value: DIALUP_LINE; Name: SDialupLine),
    (Value: LOCAL_HANDSET; Name: SLocalHandset),
    (Value: EXTERNAL_MICROPHONE; Name: SExternalMicrophone),
    (Value: INTERNAL_SPEAKER; Name: SInternalSpeaker));

  PlayDeviceMap: array[0..2] of TIdentMapEntry = (
    (Value: DIALUP_LINE; Name: SDialupLine),
    (Value: LOCAL_HANDSET; Name: SLocalHandset),
    (Value: INTERNAL_SPEAKER; Name: SInternalSpeaker));

  RecordDeviceMap: array[0..2] of TIdentMapEntry = (
    (Value: DIALUP_LINE; Name: SDialupLine),
    (Value: LOCAL_HANDSET; Name: SLocalHandset),
    (Value: EXTERNAL_MICROPHONE; Name: SExternalMicrophone));

{ TZyXEL2864 }

function TZyXEL2864.AnswerPhone: Integer;
begin
  { Check modem specified }
  if not ModemAssigned(True) then
  begin
    Result := FAIL;
    Exit;
  end;
  { Send command }
  Result := Modem.Command('AT+VLS=2', 'OK|CONNECT*');
  { Check result }
  if Result and VMA_USER <> VMA_USER then
  begin
    Result := VMA_ERROR;
    Exit;
  end;
  if Result = VMA_USER_2 then
  begin
    Result := VMA_CONNECT;
    Exit;
  end;
  Result := VMA_OK;
end;

class function TZyXEL2864.GetCompression(const Name: string): Integer;
begin
  Result := IdentToIntDef(Name, StrToIntDef(Name, -1), CompressionMap);
end;

class function TZyXEL2864.GetCompressionName(Compression: Integer): string;
begin
  Result := IntToIdentDef(Compression, IntToStr(Compression), CompressionMap);
end;

class function TZyXEL2864.GetCompressionsList(Proc: TGetStrProc): Integer;
begin
  Result := GetIdentsList(CompressionMap, Proc);
end;

class function TZyXEL2864.GetDevice(const Name: string): Integer;
begin
  Result := IdentToIntDef(Name, StrToIntDef(Name, -1), DeviceMap);
end;

class function TZyXEL2864.GetDeviceName(Device: Integer): string;
begin
  Result := IntToIdentDef(Device, IntToStr(Device), DeviceMap);
end;

class function TZyXEL2864.GetDevicesList(Proc: TGetStrProc): Integer;
begin
  Result := GetIdentsList(DeviceMap, Proc);
end;

class function TZyXEL2864.GetPlayDevicesList(Proc: TGetStrProc): Integer;
begin
  Result := GetIdentsList(PlayDeviceMap, Proc);
end;

class function TZyXEL2864.GetRecordDevicesList(Proc: TGetStrProc): Integer;
begin
  Result := GetIdentsList(RecordDeviceMap, Proc);
end;

function TZyXEL2864.Init: Integer;
begin
  DebugMsg(L_MESG, SInit);
  { Check modem specified }
  if not ModemAssigned(True) then
  begin
    Result := FAIL;
    Exit;
  end;
  { Update modem state }
  Modem.State := INITIALIZING;
  try
    { AT&H3 - Hardware CTS/RTS flow control.
      AT&R1 - Modem assumes RTS always ON. }
    if Modem.Command('AT&H3&R1', 'OK') <> VMA_USER_1 then
    begin
      DebugMsg(L_WARN, SFlowControlError);
      Result := FAIL;
      Exit;
    end;
    { Set extended verbal result messages }
    if Modem.Command('ATX5', 'OK') <> VMA_USER_1 then
    begin
      DebugMsg(L_WARN, SExResultCodesError);
      Result := FAIL;
      Exit;
    end;
    { Switch to IS 101 mode }
    if Modem.Command('ATS48.5=1', 'OK') <> VMA_USER_1 then
      DebugMsg(L_WARN, 'Couldn''t set IS 101 compatible mode');
    { ATS40.3=1 - Enable distincitve ring type 1 (RING)
      ATS40.4=1 - Enable distincitve ring type 2 (RING 1)
      ATS40.5=1 - Enable distincitve ring type 3 (RING 2)
      ATS40.6=1 - Enable distincitve ring type 4 (RING 3) }
    if Modem.Command('ATS40.3=1S40.4=1S40.5=1S40.6=1', 'OK') <> VMA_USER_1 then
      DebugMsg(L_WARN, SDistinctiveRingError);
  finally
    Modem.State := IDLE;
  end;
  { Init complete }
  Result := OK;
end;

function TZyXEL2864.InitVoice: Integer;
var
  TransmitGain, ReceiveGain: Integer;
begin
  { Check modem specified }
  if not ModemAssigned(True) then
  begin
    Result := FAIL;
    Exit;
  end;
  { Update modem state }
  Modem.State := INITIALIZING;
  try
    { AT+VIT=100 - Set inactivity timer to 10 seconds }
    if Modem.Command('AT+VIT=100', 'OK') <> VMA_USER_1 then
      DebugMsg(L_WARN, 'Voice init failed, continuing');
    { AT+VDD=x,y - Set DTMF tone detection threshold and duration detection }
     if Modem.CommandFmt('AT+VDD=%d,%d', 'OK', [Profile.DTMFSens * 15 div 100, Profile.DTMFLen div 5]) <> VMA_USER_1 then
       DebugMsg(L_WARN, 'Setting DTMF preferences didn''t work');
    { AT+VSD=x,y - Set silence threshold and duration }
    if Modem.CommandFmt('AT+VSD=%d,%d', 'OK', [Profile.RecSilenceSens * 31 div 100, Profile.RecSilenceLen]) <> VMA_USER_1 then
      DebugMsg(L_WARN, SRecordingPreferencesError);
    { AT+VGT - Set the transmit gain for voice samples }
    TransmitGain := Profile.TransmitGain;
    if TransmitGain = -1 then TransmitGain := 50;
    if Modem.CommandFmt('AT+VGT=%d', 'OK', [TransmitGain * 144 div 100 + 56]) <> VMA_USER_1 then
      DebugMsg(L_WARN, 'Setting transmit gain didn''t work');
    { AT+VGR - Set receive gain for voice samples. }
    ReceiveGain := Profile.ReceiveGain;
    if ReceiveGain = -1 then ReceiveGain := 50;
    if Modem.CommandFmt('AT+VGR=%d', 'OK', [ReceiveGain * 144 div 100 + 56]) <> VMA_USER_1 then
      DebugMsg(L_WARN, 'Setting receive gain didn''t work');
    { Delay after ringback or before any ringback before modem assumes
      phone has been answered }
    if Modem.CommandFmt('AT+VRA=%d+VRN=%d', 'OK', [Profile.RingbackGoesAway, Profile.RingbackNeverCame]) <> VMA_USER_1 then
      DebugMsg(L_WARN, SRingbackTimersError);
  finally
    Modem.State := IDLE;
  end;
  { Success }
  Result := OK;
end;

class function TZyXEL2864.ModemName: string;
begin
  Result := 'ZyXEL 2864';
end;

class function TZyXEL2864.ModemRMDName: string;
begin
  Result := 'ZyXEL 2864';
end;

function TZyXEL2864.SetCompression(var Compression, Speed, Bits: Integer): Integer;
begin
  { Check modem specified }
  if not ModemAssigned(True) then
  begin
    Result := FAIL;
    Exit;
  end;
  { The default mode for ZyXEL modems is ADPCM }
  if Compression = 0 then Compression := COMPRESSION_ZYXEL2864_ADPCM_2;
  { 7200, 8000, 9600, 11025 Hz is recording freqs supported by ZyXEL 2864 }
  if Speed = 0 then Speed := 9600;
  if (Speed <> 7200) and (Speed <> 8000) and (Speed <> 9600) and (Speed <> 11025) then
  begin
    DebugMsgFmt(L_WARN, SIllegalSampleRate, [Speed]);
    Result := FAIL;
    Exit;
  end;
  { Setup compression }
  case Compression of
    COMPRESSION_ZYXEL2864_ADPCM_2:
      { 2 bit ADPCM }
      begin
        Bits := 2;
        if Modem.CommandFmt('AT+VSM=2,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_ADPCM_3:
      { 3 bit ADPCM }
      begin
        Bits := 3;
        if Modem.CommandFmt('AT+VSM=3,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_ADPCM_30:
      { New 3 bit ADPCM }
      begin
        Bits := 3;
        if Modem.CommandFmt('AT+VSM=30,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_ADPCM_4:
      { 4 bit ADPCM }
      begin
        Bits := 4;
        if Modem.CommandFmt('AT+VSM=4,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_DVIADPCM_4:
      { 4 bit DVI ADPCM }
      begin
        Bits := 4;
        if Modem.CommandFmt('AT+VSM=40,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_ALAW_PCM:
      { A-law PCM }
      begin
        Bits := 8;
        if Modem.CommandFmt('AT+VSM=80,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    COMPRESSION_ZYXEL2864_MULAW_PCM:
      { Mu-law PCM }
      begin
        Bits := 8;
        if Modem.CommandFmt('AT+VSM=81,%d', 'OK', [Speed]) <> VMA_USER_1 then
        begin
          Result := VMA_FAIL;
          Exit;
        end;
      end;
    else
      begin
        DebugMsgFmt(L_WARN, SIllegalCompressionMethod, [Compression]);
        Modem.LastError := VME_ILLEGAL_COMPRESSION;
        Result := FAIL;
        Exit;
      end;
  end;
  Result := OK;
  Exit;
end;

function TZyXEL2864.SetDevice(Device: Integer): Integer;
begin
  { Check modem specified }
  if not ModemAssigned(True) then
  begin
    Result := FAIL;
    Exit;
  end;
  { Set device }
  case Device of
    NO_DEVICE:
      begin
        Modem.Command('AT+VLS=0', 'OK');
        Result := OK;
        Exit;
      end;
    LOCAL_HANDSET:
      begin
        Modem.Command('AT+VLS=1', 'OK');
        Result := OK;
        Exit;
      end;
    DIALUP_LINE:
      begin
        if Modem.Command('AT+VLS=2', 'OK|CONNECT*') = VMA_USER_2 then
        begin
          Result := VMA_CONNECT;
          Exit;
        end;
        Result := OK;
        Exit;
      end;
    EXTERNAL_MICROPHONE:
      begin
        Modem.Command('AT+VLS=8', 'OK');
        Result := OK;
        Exit;
      end;
    INTERNAL_SPEAKER:
      begin
        Modem.Command('AT+VLS=16', 'OK');
        Result := OK;
        Exit;
      end;
  end;
  { Given device unknown }
  DebugMsgFmt(L_WARN, SUnknownDevice,[Device]);
  Modem.LastError := VME_UNKNOWN_DEVICE;
  Result := FAIL;
end;

initialization
  RegisterModemDriver(TZyXEL2864.ModemName, TZyXEL2864);

end.
